package org.infinispan.server.configuration.resp;

import java.util.HashMap;
import java.util.Map;

/**
 * @author William Burns
 * @since 14.0
 */
public enum Element {
   //must be first
   UNKNOWN(null),

   RESP_CONNECTOR,
   AUTHENTICATION,
   ENCRYPTION;

   private static final Map<String, Element> ELEMENTS;

   static {
      final Map<String, Element> map = new HashMap<>(8);
      for (Element element : values()) {
         final String name = element.name;
         if (name != null) {
            map.put(name, element);
         }
      }
      ELEMENTS = map;
   }

   private final String name;

   Element(final String name) {
      this.name = name;
   }

   Element() {
      this.name = name().toLowerCase().replace('_', '-');
   }

   public static Element forName(final String localName) {
      final Element element = ELEMENTS.get(localName);
      return element == null ? UNKNOWN : element;
   }

   @Override
   public String toString() {
      return name;
   }
}
